// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_http_client/src/middleware.dart';
import 'package:wyatt_http_client/src/middleware_client.dart';
import 'package:wyatt_http_client/src/middlewares/default_middleware.dart';
import 'package:wyatt_http_client/src/models/middleware_request.dart';
import 'package:wyatt_http_client/src/pipeline.dart';

class RefreshTokenMiddleware extends Middleware {
  final String authorizationEndpoint;
  final String tokenEndpoint;

  String? accessToken;
  String? refreshToken;

  Middleware innerClientMiddlewares;

  RefreshTokenMiddleware({
    required this.authorizationEndpoint,
    required this.tokenEndpoint,
    Middleware? innerClientMiddlewares,
    super.child,
  }) : innerClientMiddlewares = innerClientMiddlewares ?? DefaultMiddleware();

  @override
  String getName() => 'RefreshTokenMiddleware';

  @override
  MiddlewareRequest onRequest(MiddlewareRequest request) {
    print(
      'RefreshToken::OnRequest: accessToken: $accessToken',
    );
    if (accessToken == null) {
      // Refresh token
      final pipeline = Pipeline().addMiddleware(innerClientMiddlewares);
      print(pipeline.getLogic());
      final client = MiddlewareClient(
        pipeline,
        inner: getClient(),
      );
      final _ = client.post(Uri.parse(tokenEndpoint));
    }
    var newReq = request.unfreezedRequest;
    final mutation = {
      'authorization': accessToken ?? '',
    };
    Map<String, String>? headers = newReq.headers;
    if (headers != null) {
      headers.addAll(mutation);
    } else {
      headers = mutation;
    }
    newReq = newReq.copyWith(headers: headers);
    request.updateUnfreezedRequest(newReq);

    return super.onRequest(request);
  }
}
