// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_http_client/src/middleware.dart';
import 'package:wyatt_http_client/src/middlewares/default_middleware.dart';

class Pipeline {
  final Middleware _middleware;

  Pipeline() : _middleware = DefaultMiddleware();

  Pipeline addMiddleware(Middleware middleware) {
    _middleware.addChild(middleware);
    return this;
  }

  Middleware get middleware {
    return _middleware;
  }

  Pipeline operator +(Pipeline other) {
    final copy = _middleware.deepCopy()..addChild(other.middleware);
    return Pipeline()..addMiddleware(copy);
  }

  String getLogic() {
    final req = <String>[];
    final res = <String>[];
    Middleware? m = _middleware;
    while (m != null) {
      if (m is! DefaultMiddleware) {
        req.add('$m');
        res.insert(0, '$m');
      }
      m = m.child;
    }
    return '[Req] -> ${req.join(' -> ')}\n[Res] -> ${res.join(' -> ')}';
  }

  @override
  String toString() {
    return middleware.toString();
  }
}
